

------ SIR BASIC -- version 1 
function createObserverChart (terrame_object, plotLabels)
	chart = Observer {
		subject     = terrame_object,
		attributes  = plotLabels.attributes,
		curveLabels = plotLabels.attributes,
		title       = plotLabels.title,
		xLabel      = plotLabels.xLabel,
		yLabel      = plotLabels.yLabel,
		type = "chart"
	}
end

----
function createPlot (model)

	model.plot = Cell (model.variables)
	createObserverChart (model.plot, model.plotLabels)
	model.plot:notify(0) 

end
------------------
function createTimer (model)

	model.timer = Timer {
		Event{time = 1, period = 1, action = function(e)
			changes(model)    -- 
			model.plot:notify(e:getTime()) 
		end}
	}
end

--------------------

function calcParameters (model)

	model.constants.infectionRate = model.constants.contactsPerInfectionDay * model.constants.contagionStrength / model.constants.total

	model.constants.recoveryRate = 1 / model.constants.infectiousPeriod

	model.constants.R0 = (model.constants.infectionRate * model.constants.total) / model.constants.recoveryRate

	print ("R0   = ", model.constants.R0)
end 


-------------------

function changes (model)

	new_infected  = model.constants.infectionRate*model.variables.susceptible*model.variables.infected

	new_recovered = model.constants.recoveryRate*model.variables.infected

	if new_infected > model.variables.susceptible then
		new_infected = model.variables.susceptible
	end

	
	model.variables.susceptible = model.variables.susceptible - new_infected 

	model.variables.infected    = model.variables.infected + new_infected - new_recovered

	model.variables.recovered   = model.variables.recovered + new_recovered

end

-----------------

function run (model, time)
	model.timer:execute (time)
end

-----------
function SystemDynamicsModel(model)

	-- create Plot
	createPlot (model)

    -- set constants
    calcParameters (model)

	-- timer
	createTimer (model)

	return model
end
---------
model =  SystemDynamicsModel {
	plotLabels = {
		attributes = {"susceptible",  "infected", "recovered"},
		title      = {"SIR model"},
		xLabel     = {"days"},
		yLabel     = {"people"}
	},
	variables     = { susceptible = 999, infected = 1, recovered = 0},
	constants     = { total = 1000, contactsPerInfectionDay = 1, contagionStrength = 0.4, infectiousPeriod = 4,
					  infectionRate, recoveryRate, R0}
}


-----
run (model, 100) 




